package com.bank;

import junit.framework.TestCase;

import java.util.Hashtable;

public class AccountServiceTest extends TestCase implements TransactionHelper
{
    private TestForAccountService _testForAccountService;

    public void testGetAccount()
    {
        _testForAccountService.testGetAccount();
    }

    public void testTransferFund()
    {
        _testForAccountService.testTransferFund();
    }

    public void testTransferMoreThanBalance()
    {
        _testForAccountService.testTransferMoreThanBalance();
    }

    public void testTransferOverMinimumBalance()
    {
        _testForAccountService.testTransferOverMinimumBalance();
    }

    public void rollbackTransaction()
    {
    }

    class AccountDaoMock implements AccountDao
    {
        Hashtable accounts = new Hashtable();
        // This is where the hand tossed mock may be getting a bit more complex than it should be.
        // We may benefit from using a framework like EasyMock or JMock for this.

        public Account getAccount(int id)
        {
            Account account = (Account) accounts.get(id);

            if (account == null)
            {
                account = new Account();
                account.setId(id);
                account.setBalance(id * 5000);
            }

            return account;
        }

        public void save(Account account)
        {
            String error = account.validate();

            if (error != null)
            {
                accounts.clear(); // as if to rollback transaction and clear what's in memory
                throw new AccountSaveException(error);
            }
            else
            {
                accounts.put(account.getId(), account);
            }
        }
    }

    protected void setUp() throws Exception
    {
        AccountService accountService = new AccountService();
        accountService.setAccountDao(new AccountDaoMock());

        _testForAccountService = new TestForAccountService(accountService, this);
    }
}
