package com.bank;

import static junit.framework.Assert.*;

public class TestForAccountService
{
    private AccountService _accountService;
    private TransactionHelper _transactionHelper;

    public TestForAccountService(AccountService accountService, TransactionHelper transactionHelper)
    {
        _accountService = accountService;
        _transactionHelper = transactionHelper;
    }

    public void testGetAccount()
    {
        Account account = _accountService.getAccount(1);
        assertEquals(1, account.getId());
        assertEquals(5000, account.getBalance());
    }

    public void testTransferFund()
    {
        Account account1 = _accountService.getAccount(1);
        Account account2 = _accountService.getAccount(2);

        assertEquals(5000, account1.getBalance());
        assertEquals(10000, account2.getBalance());

        _accountService.transferFund(1,  2,  1000);

        account1 = _accountService.getAccount(1);
        account2 = _accountService.getAccount(2);

        assertEquals(4000, account1.getBalance());
        assertEquals(11000, account2.getBalance());
    }

    public void testTransferMoreThanBalance()
    {
        Account account1 = _accountService.getAccount(1);
        Account account2 = _accountService.getAccount(2);

        assertEquals(5000, account1.getBalance());
        assertEquals(10000, account2.getBalance());

        try
        {
            _accountService.transferFund(1,  2,  10000);
            fail("InsufficientBalanceException  expected...");
        }
        catch(InsufficientBalanceException ex)
        {
            // OK
        }

        account1 = _accountService.getAccount(1);
        account2 = _accountService.getAccount(2);

        assertEquals(5000, account1.getBalance());
        assertEquals(10000, account2.getBalance());
    }

    public void testTransferOverMinimumBalance()
    {
        Account account1 = _accountService.getAccount(1);
        Account account2 = _accountService.getAccount(2);

        assertEquals(5000, account1.getBalance());
        assertEquals(10000, account2.getBalance());

        try
        {
            _accountService.transferFund(1,  2,  4001);
            fail("AccountSaveException expected...");
        }
        catch(AccountSaveException ex)
        {
            // OK
        }

        _transactionHelper.rollbackTransaction();

        account1 = _accountService.getAccount(1);
        account2 = _accountService.getAccount(2);

        assertEquals(5000, account1.getBalance());
        assertEquals(10000, account2.getBalance());
    }
}
